#include "oslAnimLib.h"

/* Description: This loads the animation's frames using a sprite sheet           */
/* Usage: Call this before any other OSL_ANIMATION call - uses interval + offset */
/* Example:                                                                      */
/*       animWalkingLeft.Load("Images/Resources/Huge Explosion",6,0,0,32,32,true)*/
/*   This will load 6 frames, starting at 0,0 each with the dimensions 32x32     */
/*                            going across horizontally                          */
void OSL_ANIMATION::Load(const std::string & filename, const int numframes, const int startx, const int starty, const int width, const int height, const bool horizontal)
{
  Unload();

  char cFilename[1036];
  strncpy(cFilename, (filename + ".png").c_str(), 1036);

  sSpriteSheet = oslLoadImageFile(cFilename, OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888); // load it
  oslAssert(sSpriteSheet); // verify it loaded correctly

  int StartX = startx; // these are used for if the sprite sheet row/coloumn end is reached
  int StartY = starty;
  int tmp = 0;   // temporary var used for the rows and coloumns in the animation
  iWidth = width;
  iHeight = height;
  iNumFrames = numframes;

  for (int i = 0; i < numframes; i++)
  {
    vFrames.push_back(oslCreateImageTileSize(sSpriteSheet, StartX + (horizontal ? (tmp * width) : 0), StartY + (horizontal ? 0 : (tmp * height)),
                                             width, height));
    oslAssert(vFrames[i]);

    tmp++;

    // now we check and fix any sprite sheet sprite row/coloumn endings
    if (horizontal)
    {
      if (StartX + ((tmp) * width) >= sSpriteSheet->stretchX)
      {
        StartY += height; // go down a row
        StartX = startx;
        tmp = 0;
      }
    }
    else
    {
      if (StartY + ((tmp) * height) >= sSpriteSheet->stretchY)
      {
        StartX += width; // move over a coloumn
        StartY = starty;
        tmp = 0;
      }
    }
  }

  sFilename = filename;
}

/* Description: This loads the animation's frames using a multiple images        */
/* Usage: Call this before any other OSL_ANIMATION call                          */
/* Example:                                                                      */
/*       animWalkingLeft.Load("Images/Resources/Huge Explosion",6)               */
/*   This will load 6 frames named consecutively located at                      */
/*              ImagesResources/Huge Explosion[0 - 6 here].png                   */
void OSL_ANIMATION::Load(std::string filename, const int numframes)
{
  Unload();

  iNumFrames = numframes;

  char tempFilename[500];

  for (int i = 0; i < iNumFrames; i++)
  {
    snprintf(tempFilename, 500, "%s%i.png",filename.c_str(),i+1);

    vFrames.push_back(oslLoadImageFile(tempFilename, OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888)); // add it
    oslAssert(vFrames[i]);
  }

  iWidth = vFrames[0]->stretchX;
  iHeight = vFrames[0]->stretchY;
  sFilename = filename; // just set it back to how it was...
}

/* Description: This unloads the animation's frames                              */
/* Usage: Call this when leaving a state, exiting the game, etc.                 */
void OSL_ANIMATION::Unload()
{
  for (int i = 0; i < (int)vFrames.size(); i++)
  {
    if (vFrames[i])
      oslDeleteImage(vFrames[i]);

    vFrames[i] = NULL;
  }
  vFrames.clear();

  if (sSpriteSheet)
    oslDeleteImage(sSpriteSheet);

  sSpriteSheet = NULL;
}

/* Description: This initializes alot of members                                 */
/* Usage: Call this after you load the animation or to reset the animation       */
/* Example:                                                                      */
/*         animWalkingLeft.Initialize(0,0,32,32,0,10,20,false,true,false)        */
/*                                                                               */
/* This makes animWalkingLeft X and Y at (0,0).  It's width and height 32, it's  */
/* starting frame at 0, ending frame at 10. The animation delay at 20, it's angle*/
/* at 0 degrees, it won't play after being initialized, it will loop when played */
/* and it won't play reversed.                                                   */
void OSL_ANIMATION::Initialize(const int x, const int y, const int width, const int height, const int startFrame, const int endFrame,
                const int delay, const bool play, const bool loop, const bool reverse)
{
  if (width != -1) iWidth = width;
  if (height != -1) iHeight = height;

  iCenterX = iWidth/2; // middle of image
  iCenterY = iHeight/2;// "

  iX = x;
  iY = y;

  iStartFrame = startFrame;
  iEndFrame = endFrame;

  iDelay = delay;
  iDelayCount = 0;

  fAngle = 0.0f;

  bPlay = play;
  bLoop = loop;
  bReverse = reverse;

  if (bReverse)
    iCurrentFrame = iEndFrame;
  else
    iCurrentFrame = iStartFrame;
}

/* Description: This does all the drawing for the animationn                     */
/* Usage: Call this in your main loop for when you want the animation to be drawn*/
/* Example:                                                                      */
/*            if ((bool) bWalkingLeft)                                           */
/*            {                                                                  */
/*              animWalkingLeft.Draw();                                          */
/*            }                                                                  */
void OSL_ANIMATION::Draw()
{
  if (bPlay)
  {
    if (!vFrames[iCurrentFrame])
      oslDebug("%s\nvFrames[%i] not loaded!",sFilename.c_str(),iCurrentFrame);

    oslDrawImage(vFrames[iCurrentFrame]);
  }
}

/* Description: This does all the thinking for the animation                     */
/* Usage: Call this in your main loop for when you want the animation to update  */
/* Example:                                                                      */
/*            if ((bool) bWalkingLeft)                                           */
/*            {                                                                  */
/*              animWalkingLeft.Update();                                        */
/*            }                                                                  */
void OSL_ANIMATION::Update()
{
  if (bPlay)
  {
    // if the frames are out of whack :S
    if (iEndFrame < iStartFrame)
      oslDebug("When updating an animation, it appears it's end frame (%i) is LESS THEN it's start frame (%i)\n\nwtf is wrong with you?", iEndFrame, iStartFrame);

    for (int i = iStartFrame; i <= iEndFrame; i++)
    {
      if (!vFrames[i])
        oslDebug("%s\nvFrames[%i] not loaded!",sFilename.c_str(),i);

      // update animation frames' value's
      vFrames[i]->centerX = iCenterX;
      vFrames[i]->centerY = iCenterY;
      vFrames[i]->x = iX;
      vFrames[i]->y = iY;
      vFrames[i]->stretchX = iWidth;
      vFrames[i]->stretchY = iHeight;
      vFrames[i]->angle = (int)fAngle;
    }

    // delay stuff
    iDelayCount++;

    if (iDelayCount >= iDelay)
    {
      if (bReverse)
      {
        iCurrentFrame--;

        if (iCurrentFrame < iStartFrame)
        {
          iCurrentFrame = iEndFrame;

          if (!bLoop)
          {
            bPlay = false;
          }
        }
      }
      else
      {
        iCurrentFrame++;

        if (iCurrentFrame > iEndFrame)
        {
          iCurrentFrame = iStartFrame;

          if (!bLoop)
          {
            bPlay = false;
          }
        }
      }

      iDelayCount = 0;
    }
  }
}
