#ifndef OSL_ANIMLIB_H
#define OSL_ANIMLIB_H

#include <iostream>         // these STLs must go first else you get all sorts of errors...
#include <vector>
#include <string>
#include <oslib/oslib.h>

/*
               oslAnimLib

   This is a library to aid in animating
  multiple images or a sprite sheet in the
  advanced 2D graphics library 'OSlib' for
 the PSP.  Inspired by animLib, same concept
    but for, and in, Lua by Grimfate126.

     My only request is that if you
    use this in your project, you give
some credit to myself for this particular
         mechanic (animations)

 Let me know if this has been of use to you
       and possibly request another
      useful class you'd like to see

                                    ~ SG57
*/

// Check out the sample for correct usage of this library
//
// If your using C, add -lstdc++ to your LIBS makefile line
// and change your source file's extension to .cpp INSTEAD of .c - that's it!
class OSL_ANIMATION
{
  private:
    OSL_IMAGE *sSpriteSheet;   // since oslCreateImageTile references the image, we can't delete it afterwards or have it temporary
    std::vector<OSL_IMAGE*> vFrames; // dynamic array of OSL_IMAGE's
    std::string sFilename;          // file path and name for the images
    int iX, iY;                // coordintes of the animation
    int iWidth, iHeight;       // dimensions of the animation
    int iCenterX, iCenterY;    // the 'pole' the image is rotated around
    int iNumFrames;            // number of frames
    int iCurrentFrame;         // current frame in the animation were on
    int iStartFrame;           // frame that the animation starts on
    int iEndFrame;             // frame that the animation ends on
    int iDelay;                // delay count until we go to the next frame in the animation
    int iDelayCount;           // internally used - this value is incremented til' it reaches iDelay then resets
    float fAngle;              // angle in degrees the animation is rotated
    bool bPlay;                // flag to tell if we should play the animation
    bool bLoop;                // flag to tell if we should loop the animation
    bool bReverse;             // flag to tell if we should play animation reversed

  public:
    // Constructor
    OSL_ANIMATION() : sFilename("/"), iX(0), iY(0), iWidth(0), iHeight(0), iCenterX(0), iCenterY(0), iNumFrames(0), iCurrentFrame(0), iEndFrame(0), iDelay(0),
                      iDelayCount(0), fAngle(0.0f), bPlay(false), bLoop(false), bReverse(false)
    {
      sSpriteSheet = NULL;
      vFrames.clear();
    }
    // Destructor
    ~OSL_ANIMATION()
    {
      Unload();
    }

    /*
                               DO
      These functions actually do stuff, not set/get values
    */

    /* Description: This loads the animation's frames using a sprite sheet           */
    /* Usage: Call this before any other OSL_ANIMATION call - uses interval + offset */
    /* Example:                                                                      */
    /*       animWalkingLeft.Load("Images/Resources/Huge Explosion",6,0,0,32,32,true)*/
    /*   This will load 6 frames, starting at 0,0 each with the dimensions 32x32     */
    /*                            going across horizontally                          */
    void Load(const std::string & filename, const int numframes, const int startx, const int starty, const int width, const int height, const bool horizontal);

    /* Description: This loads the animation's frames using a multiple images        */
    /* Usage: Call this before any other OSL_ANIMATION call                          */
    /* Example:                                                                      */
    /*       animWalkingLeft.Load("Images/Resources/Huge Explosion",6)               */
    /*   This will load 6 frames named consecutively located at                      */
    /*              ImagesResources/Huge Explosion[0 - 6 here].png                   */
    void Load(std::string filename, const int numframes);
    
    /* Description: This unloads the animation's frames                              */
    /* Usage: Call this when leaving a state, exiting the game, etc.                 */
    void Unload();
    
    /* Description: This initializes alot of members                                 */
    /* Usage: Call this after you load the animation or to reset the animation       */
    /* Example:                                                                      */
    /*         animWalkingLeft.Initialize(0,0,32,32,0,10,20,false,true,false)        */
    /*                                                                               */
    /* This makes animWalkingLeft X and Y at (0,0).  It's width and height 32, it's  */
    /* starting frame at 0, ending frame at 10. The animation delay at 20, it's angle*/
    /* at 0 degrees, it won't play after being initialized, it will loop when played */
    /* and it won't play reversed.                                                   */
    void Initialize(const int x, const int y, const int width, const int height, const int startFrame, const int endFrame,
                    const int delay, const bool play, const bool loop, const bool reverse);

    /* Description: This does all the drawing for the animationn                     */
    /* Usage: Call this in your main loop for when you want the animation to be drawn*/
    /* Example:                                                                      */
    /*            if ((bool) bWalkingLeft)                                           */
    /*            {                                                                  */
    /*              animWalkingLeft.Draw();                                          */
    /*            }                                                                  */
    void Draw();
    
    /* Description: This does all the thinking for the animation                     */
    /* Usage: Call this in your main loop for when you want the animation to update  */
    /* Example:                                                                      */
    /*            if ((bool) bWalkingLeft)                                           */
    /*            {                                                                  */
    /*              animWalkingLeft.Update();                                        */
    /*            }                                                                  */
    void Update();

    /*
                          SET
      These functions set private members' values
    */

    /* Description: This sets the bPlay boolean to true meaning play the animation   */
    /* Usage: Pass this when an event occurs and you want to play the animation      */
    /* Example: myAnimation.setFilename("Resources/Animations/Huge Explosion")       */
    inline void Play()
    {
      bPlay = true;
    }

    /* Description: This sets y-coordinate as to where to draw the animation         */
    /* Usage: Pass this an integer for where to draw the animation on the y-axis     */
    /* Example: myAnimation.setY(272)                                                */
    inline void setY(const int y)
    {
      iY = y;
    }

    /* Description: This sets x-coordinate as to where to draw the animation         */
    /* Usage: Pass this an integer for where to draw the animation on the x-axis     */
    /* Example: myAnimation.setY(480)                                                */
    inline void setX(const int x)
    {
      iX = x;
    }

    /* Description: This sets the width of the animation                             */
    /* Usage: Pass this an integer that the animation's width will be stretched to   */
    /* Example: myAnimation.setWidth(256)                                            */
    inline void setWidth(const int width)
    {
      iWidth = width;
    }

    /* Description: This sets the height of the animation                            */
    /* Usage: Pass this an integer that the animation's height will be stretched to  */
    /* Example: myAnimation.setHeight(256)                                           */
    inline void setHeight(const int height)
    {
      iHeight = height;
    }

    /* Description: This sets the center X of rotation of the animation              */
    /* Usage: Pass this an integer that the animation's rotation center X will be    */
    /* Example: myAnimation.setCenterX(128)                                          */
    inline void setCenterX(const int x)
    {
      iCenterX = x;
    }

    /* Description: This sets the center Y of rotation of the animation              */
    /* Usage: Pass this an integer that the animation's rotation center Y will be    */
    /* Example: myAnimation.setCenterY(128)                                          */
    inline void setCenterY(const int y)
    {
      iCenterY = y;
    }

    /* Description: This sets the angle of the animation                             */
    /* Usage: Pass this an integer that the animation will be rotated to (degrees)   */
    /* Example: myAnimation.setRotationAngle(180)                                    */
    inline void setRotationAngle(const float angle)
    {
      fAngle = angle;
    }

    /* Description: This sets the delay of the animation                             */
    /* Usage: Pass this an integer that the animation will delay for before going to */
    /*        the next frame in the animation (tweak this til it's right)            */
    /* Example: myAnimation.setDelay(100)                                            */
    inline void setDelay(const int delay)
    {
      iDelay = delay;
    }

    /* Description: This sets the current frame of the animation                     */
    /* Usage: Pass this an integer that the current animation frame will be displaed */
    /* Example: myAnimation.setCurrentFrame(2)                                       */
    inline void setCurrentFrame(const int frame)
    {
      if (vFrames[frame])
        iCurrentFrame = frame;
      else
        oslDebug("problem when setting the current frame to %i - it isn't loaded?", frame);
    }

    /* Description: This sets the starting frame of the animation                    */
    /* Usage: Pass this an integer that the animation will start on                  */
    /* Example: myAnimation.setStartFrame(3)                                         */
    inline void setStartFrame(const int frame)
    {
      iStartFrame = frame;
    }

    /* Description: This sets the ending frame of the animation                      */
    /* Usage: Pass this an integer that the animation will end on                    */
    /* Example: myAnimation.setEndFrame(7)                                           */
    inline void setEndFrame(const int frame)
    {
      iEndFrame = frame;
    }

    /* Description: This sets whether or not to loop the animation                   */
    /* Usage: Pass this a boolean for if the animation should loop when finished     */
    /* Example: myAnimation.setLoop(true)                                            */
    inline void setLoop(const bool loop)
    {
      bLoop = loop;
    }


    /* Description: This sets whether or not to play the animation backwards         */
    /* Usage: Pass this a boolean for if the animation should play reversedished     */
    /* Example: myAnimation.setReverse(true)                                         */
    inline void setReverse(const bool reverse)
    {
      bReverse = reverse;
    }

    /*
                     GET
      These functions get private members

      Won't demonstrate usage of the obvious ones ;)
    */
    inline const std::vector<OSL_IMAGE*> & getFrames()
    {
      return vFrames;
    }

    inline const std::string & getFilename()
    {
      return sFilename;
    }

    inline const int getY()
    {
      return iY;
    }
    inline const int getX()
    {
      return iX;
    }
    inline const int getWidth()
    {
      return iWidth;
    }
    inline const int getHeight()
    {
      return iHeight;
    }
    inline const int getCenterX()
    {
      return iCenterX;
    }
    inline const int getCenterY()
    {
      return iCenterY;
    }
    inline const float getRotationAngle()
    {
      return fAngle;
    }
    inline const int getDelay()
    {
      return iDelay;
    }
    inline const int getStartFrame()
    {
      return iStartFrame;
    }
    inline const int getEndFrame()
    {
      return iEndFrame;
    }
    inline const int getCurrentFrame()
    {
      return iCurrentFrame;
    }
    inline const int getFrameCount()
    {
      return iNumFrames;
    }

    inline const bool isPlaying()
    {
      return bPlay;
    }
    inline const bool isLooped()
    {
      return bLoop;
    }
    inline const bool isReversed()
    {
      return bReverse;
    }
};

#endif
